add abstraction for listing config files as well
[project/uci.git] / uci.h
diff --git a/uci.h b/uci.h
index e921269872a114f43611907bbea44983aba0bb51..382e745f1d58d4f0f4db79df078e320d4719da55 100644 (file)
--- a/uci.h
+++ b/uci.h
@@ -2,9 +2,9 @@
  * libuci - Library for the Unified Configuration Interface
  * Copyright (C) 2008 Felix Fietkau <nbd@openwrt.org>
  *
- * this program is free software; you can redistribute it and/or modify
- * it under the terms of the gnu lesser general public license version 2.1
- * as published by the free software foundation
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License version 2.1
+ * as published by the Free Software Foundation
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -63,6 +63,7 @@ struct uci_section;
 struct uci_option;
 struct uci_history;
 struct uci_context;
+struct uci_backend;
 struct uci_parse_context;
 
 
@@ -135,13 +136,6 @@ extern int uci_load(struct uci_context *ctx, const char *name, struct uci_packag
  */
 extern int uci_unload(struct uci_context *ctx, struct uci_package *p);
 
-/**
- * uci_cleanup: Clean up after an error
- *
- * @ctx: uci context
- */
-extern int uci_cleanup(struct uci_context *ctx);
-
 /**
  * uci_lookup: Look up an uci element
  *
@@ -156,6 +150,15 @@ extern int uci_cleanup(struct uci_context *ctx);
  */
 extern int uci_lookup(struct uci_context *ctx, struct uci_element **res, struct uci_package *package, char *section, char *option);
 
+/**
+ * uci_add_section: Add an unnamed section
+ * @ctx: uci context
+ * @p: package to add the section to
+ * @type: section type
+ * @res: pointer to store a reference to the new section in
+ */
+extern int uci_add_section(struct uci_context *ctx, struct uci_package *p, char *type, struct uci_section **res);
+
 /**
  * uci_set_element_value: Replace an element's value with a new one
  * @ctx: uci context
@@ -174,8 +177,9 @@ extern int uci_set_element_value(struct uci_context *ctx, struct uci_element **e
  * @section: section name
  * @option: option name
  * @value: value (option) or type (section)
+ * @result: store the updated element in this variable (optional)
  */
-extern int uci_set(struct uci_context *ctx, struct uci_package *p, char *section, char *option, char *value);
+extern int uci_set(struct uci_context *ctx, struct uci_package *p, char *section, char *option, char *value, struct uci_element **result);
 
 /**
  * uci_rename: Rename an element
@@ -223,23 +227,66 @@ extern int uci_commit(struct uci_context *ctx, struct uci_package **p, bool over
 
 /**
  * uci_list_configs: List available uci config files
- *
  * @ctx: uci context
  */
 extern int uci_list_configs(struct uci_context *ctx, char ***list);
 
+/** 
+ * uci_set_savedir: override the default history save directory
+ * @ctx: uci context
+ * @dir: directory name
+ */
+extern int uci_set_savedir(struct uci_context *ctx, const char *dir);
+
+/** 
+ * uci_set_savedir: override the default config storage directory
+ * @ctx: uci context
+ * @dir: directory name
+ */
+extern int uci_set_confdir(struct uci_context *ctx, const char *dir);
+
+/**
+ * uci_add_history_path: add a directory to the search path for change history files
+ * @ctx: uci context
+ * @dir: directory name
+ *
+ * This function allows you to add directories, which contain 'overlays'
+ * for the active config, that will never be committed.
+ */
+extern int uci_add_history_path(struct uci_context *ctx, const char *dir);
+
+/**
+ * uci_revert: revert all changes to a config item
+ * @ctx: uci context
+ * @p: pointer to a uci_package struct ptr (will be replaced by the revert)
+ * @section: section name (optional)
+ * @option option name (optional)
+ */
+extern int uci_revert(struct uci_context *ctx, struct uci_package **p, char *section, char *option);
+
+/**
+ * uci_parse_argument: parse a shell-style argument, with an arbitrary quoting style
+ * @ctx: uci context
+ * @stream: input stream
+ * @str: pointer to the current line (use NULL for parsing the next line)
+ * @result: pointer for the result
+ */
+extern int uci_parse_argument(struct uci_context *ctx, FILE *stream, char **str, char **result);
+
 /* UCI data structures */
 enum uci_type {
        UCI_TYPE_HISTORY = 0,
        UCI_TYPE_PACKAGE = 1,
        UCI_TYPE_SECTION = 2,
-       UCI_TYPE_OPTION = 3
+       UCI_TYPE_OPTION = 3,
+       UCI_TYPE_PATH = 4
 };
 
 enum uci_flags {
-       UCI_FLAG_STRICT =      (1 << 0), /* strict mode for the parser */
-       UCI_FLAG_PERROR =      (1 << 1), /* print parser error messages */
-       UCI_FLAG_EXPORT_NAME = (1 << 2), /* when exporting, name unnamed sections */
+       UCI_FLAG_STRICT =        (1 << 0), /* strict mode for the parser */
+       UCI_FLAG_PERROR =        (1 << 1), /* print parser error messages */
+       UCI_FLAG_EXPORT_NAME =   (1 << 2), /* when exporting, name unnamed sections */
+       UCI_FLAG_SAVED_HISTORY = (1 << 3), /* store the saved history in memory as well */
 };
 
 struct uci_element
@@ -249,6 +296,15 @@ struct uci_element
        char *name;
 };
 
+struct uci_backend
+{
+       const char *name;
+       char **(*list_configs)(struct uci_context *ctx);
+       struct uci_package *(*load)(struct uci_context *ctx, const char *name);
+       void (*commit)(struct uci_context *ctx, struct uci_package **p, bool overwrite);
+};
+
+
 struct uci_context
 {
        /* list of config packages */
@@ -257,9 +313,18 @@ struct uci_context
        /* parser context, use for error handling only */
        struct uci_parse_context *pctx;
 
+       /* backend for import and export */
+       struct uci_backend *backend;
+
        /* uci runtime flags */
        enum uci_flags flags;
 
+       char *confdir;
+       char *savedir;
+
+       /* search path for history files */
+       struct uci_list history_path;
+
        /* private: */
        int errno;
        const char *func;
@@ -269,23 +334,6 @@ struct uci_context
        int bufsz;
 };
 
-struct uci_parse_context
-{
-       /* error context */
-       const char *reason;
-       int line;
-       int byte;
-
-       /* private: */
-       struct uci_package *package;
-       struct uci_section *section;
-       bool merge;
-       FILE *file;
-       const char *name;
-       char *buf;
-       int bufsz;
-};
-
 struct uci_package
 {
        struct uci_element e;
@@ -295,8 +343,10 @@ struct uci_package
        char *path;
 
        /* private: */
+       struct uci_backend *backend;
        int n_section;
        struct uci_list history;
+       struct uci_list saved_history;
 };
 
 struct uci_section